;==========================================================================
;
;              PARAMETERS FOR THE 2-D AGN TORUS MODELS
;
;==========================================================================

;--------------------------------------------------------------------------
;                      Name of the simulation
;--------------------------------------------------------------------------
simser = 'PaperClumpy'  ; Name of the simulation series
simnr  = 'S1'           ; Nr of this simulation in the series
simweb = 'S1'           ; The name as it will appear on a website

;--------------------------------------------------------------------------
;                         The central star
;--------------------------------------------------------------------------
rstar  = 100.d0 * RS    ; Radius of the AGN
mstar  = 1d8 * MS       ; Mass of the AGN
lagn   = 1d11 * LS      ; Luminosity of the AGN
;plspec = 1.d0           ; Spectral index 
nphot  = 1000000          ; Nr of photons

;--------------------------------------------------------------------------
;                 The opacity table and frequency grid
;--------------------------------------------------------------------------
abunc    = 0.25           ; Mix carbon
mixnames = ['amorph_mix.Kappa']
mixspecs = [['carbon.Kappa','silicate.Kappa']]
mixabun  = [[abunc,(1.0-abunc)]]
fresmd = 10             ; Frequency resolution/grid mode
infile = $              ; Dust opacity input file
  ['amorph_mix.Kappa']
pll    = [-1]           ; Power law index at long wavelengths
scat   = 1              ; Include scattering (0=no scattering)?

;--------------------------------------------------------------------------
;              Grid refinement and smoothing of solution
;--------------------------------------------------------------------------
rrefine = {nlevr:3,$    ; higher resolution at level #
           nspanr:2,$   ; number of steps to resolve
           nstepr:2}    ; number of sub-steps to create
drsm   = 3d-1           ; smoothing radius
rdxsm  = 1d0            ; smoothing level

;--------------------------------------------------------------------------
;                    The Sigma(R) setup parameters
;--------------------------------------------------------------------------
rdisk  = 10 * pc       ; Pivot of power law definitions:
sig0   = 0.0            ; Sigma at rdisk (either sig0 or mdisk)
mdisk  = 1.D6 * MS      ; Mass of the disk (either sig0 or mdisk)
plsig1 =  1.0d0		; Powerlaw for Sigma(R) for R<Rdisk
plsig2 = -12.0d0	; Powerlaw for Sigma(R) for R>Rdisk
lambda = 0.55           ; Wavelength are which kappa is computed

;--------------------------------------------------------------------------
;                    Parameters for the blobbiness
;--------------------------------------------------------------------------
nblob  = 0             ; Number of blobs
sblob  = 0.05           ; Dimensionless size of blobs
iseed  = 11985          ; Seed for random number generator
istyle = 10

;--------------------------------------------------------------------------
;           Settings for H(R) (initial value, minimal value)
;--------------------------------------------------------------------------
hrdisk =  0.300000 ; Vertical pressure scale height in units of radius
plh    = 0.00           ; Powerlaw for H(R) in setup
cdens  = 1              ; Constant density in Theta direction?

;--------------------------------------------------------------------------
;                         The spatial grid 
;--------------------------------------------------------------------------
nr     = 80             ; number of points in radius
nt     = 40             ; Number of points in theta for main theta grid
ntex   = 10              ; Extra points in theta 
rin    = 0.3*pc         ; Inner radius of radial grid (--> tin is computed)
tin    = 0.d0           ; Inner temperature (--> rin is computed)
rout   = 30.0*pc        ; Outer radius of radial grid
hrgrid = 1.2*hrdisk     ; Main grid span in Theta (measured negatively from equator)
hrgmax = 0.9*!pi/2.d0   ; Total grid span in Theta (measured negatively from equator)

;--------------------------------------------------------------------------
;              OPTIONAL: define the hard-coded sizes 
;    (you may comment these lines out by putting ';' in front!)
;--------------------------------------------------------------------------
;nr_compile    = 100     ; Hard-coded maximum for nr
;nt_compile    = 60      ; Hard-coded maximum for nt
;nf_compile    = 100     ; Hard-coded maximum for nf
;nlevr_compile = 4       ; Hard-coded maximum for nlevr (refines!)
nspec_compile = 4        ; Hard-coded maximum for nspec

;
;--------------------------------------------------------------------------
;    Path for executables for RADICAL, RADMC, DIFFUSION and VERTSTRUCT
;--------------------------------------------------------------------------
xlevel   = 1
cd,current=current
for i=1,xlevel do cd,'../'
cd,current=maindir
cd,'~/'
cd,current=homedir
cd,current
bindir=maindir+'/bin/'
;
;--------------------------------------------------------------------------
;                       Paths for the system
;--------------------------------------------------------------------------
;;
;; Paths to the Kurucz data, the dust opacity code and the dust 
;; optical constants
;;
kuruczdir = homedir+'/science/kurucz/'
src_dust  = maindir+'/sources/makeopac/src/'
dustdata  = maindir+'/sources/makeopac/optconst/'
;;
;; Paths to the source directories of the four main codes
;; (only relevant when problem_compilecodes.pro is used!
;;  because the executables are moved to `bindir`)
;;
src_rayt  = maindir+'/sources/raytrace/src'
src_rdmc  = maindir+'/sources/radmc/src'
src_chop  = maindir+'/sources/chopdens/src'
src_pahc  = maindir+'/sources/pahcode/src'


