;==========================================================
;  WIDGET ROUTINE FOR VIEWING IMAGES FROM RT CALCULATION
;==========================================================


;------------------------------------------------------------------------
;                       FUNCTION: PLANCK
;------------------------------------------------------------------------
function bplanck,nnu,TT
nu=nnu*1.d0
T=TT*1.d0
cc=2.9979d10
hh=6.6262d-27
kk=1.3807e-16
n=n_elements(nu)
bpl=dblarr(n)
if TT eq 0.d0 then return,dblarr(n)
for i=0,n-1 do begin
  x=hh*nu[i]/(kk*T)
  ;print,x
  if x gt 100.0 then b=0.d0
  if x gt 1.d-3 then b=(2.d0*hh*nu[i]^3/cc^2)/(exp(x)-1.d0) $
    else b=2.0*nu[i]^2*kk*T/cc^2
  bpl[i]=b
endfor
return,bpl
end



;----------------------------------------------------------
; COMMAND DO TELESCOPE IMAGE
;----------------------------------------------------------
pro vi_radical_do_image,angle,nx,ny,xsize,ysize,xoff,yoff,$
                        phioff,ifreq,star,background=background,$
;                        bgom=bgom,bgpl=bgpl
                        bgtmp=bgtmp,bgdil=bgdil,code=code
;
; Make command file
;
openw,1,'command'
printf,1,'image'
printf,1,'angle'
printf,1,strcompress(string(angle),/remove_all)
printf,1,'npix'
printf,1,strcompress(string(nx),/remove_all),' ',strcompress(string(ny),/remove_all)
printf,1,'imagesize'
printf,1,strcompress(string(xsize,format='(E10.3)'),/remove_all),' ',$
         strcompress(string(ysize,format='(E10.3)'),/remove_all)
printf,1,'cm'
printf,1,'offset'
printf,1,strcompress(string(xoff,format='(E10.3)'),/remove_all),' ',$
         strcompress(string(yoff,format='(E10.3)'),/remove_all),' ',$
         strcompress(string(phioff,format='(E10.3)'),/remove_all)
if ifreq ge 1 then begin
    printf,1,'ifreq'
    printf,1,strcompress(string(ifreq),/remove_all)
 endif
if star ne 0 then begin
    printf,1,'addstar'
endif
close,1
;
; If requested, make a background emission
;
if keyword_set(background) then begin
    ;;
    ;; For the moment this is disabled because I do not know
    ;; if RAYTRACE can handle this.
    ;;
    print,'Background inclusion is temporarily disabled.'
    print,'Check if RAYTRACE can do this, and then enable it again.'
    stop
    ;;
    ;; Defaults
    ;;
    if not keyword_set(bgtmp) then bgtmp = 3.d0
    if not keyword_set(bgdil) then bgdil = 1.d0
    ;;
    ;; Check the radical.inp file if the outer boundary condition = 3
    ;;
    spawn,'cp -f radical.inp radical.inp_bkviewimage'
    spawn,'cp -f radical.inp radical.inp_00'
    str= ' '
    openr,1,'radical.inp'
    openw,2,'radical.inp_00'
    for i=1,16 do begin
        readf,1,str
        printf,2,str
    endfor
    readf,1,str
    printf,2,'3'
    for i=1,20 do begin
        readf,1,str
        printf,2,str
    endfor
    close,1
    close,2
    spawn,'mv -f radical.inp_00 radical.inp'
    ;;
    ;; Make background
    ;;
    print,'With Background!'
    openr,1,'frequency.inp'
    nf=0
    readf,1,nf
    freq=dblarr(nf)
    readf,1,freq
    close,1
    openw,1,'interstellfield.inp'
    printf,1,nf
;    for inu=0,nf-1 do printf,1,bgom*(freq[inu]/3d14)^bgpl   ; OLD STUFF
    for inu=0,nf-1 do printf,1,bgdil*bplanck(freq[inu],bgtmp)
    close,1
endif
;
; Call RADICAL/RAYTRACE
;
if not keyword_set(code) then code=2
case code of
    1: spawn,"radical"
    2: spawn,"raytrace"
endcase
;
; Finish
;
if keyword_set(background) then begin
    spawn,'rm -f interstellfield.inp'
    spawn,'cp -r radical.inp_bkviewimage radical.inp'
endif
end


;-----------------------------------------------------------------
;              READ THE RECTANGULAR TELESCOPE IMAGE
;-----------------------------------------------------------------
function vi_read_recimage,ext=ext
nx=0
ny=0
nf=0
sizepix_x=0.d0
sizepix_y=0.d0
if not keyword_set(ext) then begin
   filename='image.dat'
endif else begin
   filename='image_'+strcompress(string(ext),/remove_all)+'.dat'
endelse
str=findfile(filename)
if(str(0) ne filename) then begin
    print,'Sorry, cannot find ',filename
    print,'Presumably radical exited without succes.'
    print,'See above for possible error messages of radical!'
    stop
endif
openr,1,filename
readf,1,nx,ny,nf
readf,1,sizepix_x,sizepix_y
image=fltarr(nx,ny)
tau=fltarr(nx,ny)
readf,1,image
close,1
if not keyword_set(ext) then begin
   filename='imtau.dat'
endif else begin
   filename='imtau_'+strcompress(string(ext),/remove_all)+'.dat'
endelse
str=findfile(filename)
if(str(0) eq filename) then begin
openr,1,filename
idum1=0
idum2=0
idum3=0
readf,1,idum1,idum2,idum3
if(idum1 ne nx) then stop
if(idum2 ne ny) then stop
if(idum3 ne nf) then stop
dum1=0
dum2=0
readf,1,dum1,dum2
readf,1,tau
close,1
endif
return,{nx:nx,ny:ny,nrfr:nf,sizepix_x:sizepix_x,sizepix_y:sizepix_y,$
        image:image,tau:tau}
end


;-----------------------------------------------------------------
;                   PLOT THE RECTANGULAR IMAGE
;-----------------------------------------------------------------
pro vi_plot_recimage,a,pc=pc,au=au,contour=contour,nlevels=nlevels,$
        noimage=noimage,pos=pos,thrcol=thrcol,thres=thres,$
        saturate=saturate,xticklen=xticklen,yticklen=yticklen,$
        xminor=xminor,yminor=yminor,plottau=plottau,dxs=dxs,_EXTRA=_extra
common colcodes,green,yellow,red,blue,black,white
common colors,r_orig,g_orig,b_orig,r_curr,g_curr,b_curr
;
; Interpreting the input
;
distscale=1.d0
distunit='cm'
if keyword_set(pc) then begin
    distscale = 3.24073473938d-19
    distunit  = 'pc'
endif
if keyword_set(au) then begin
    distscale = 6.68449197861d-14
    distunit  = 'AU'
endif
if not keyword_set(nlevels) then nlevels=10
if not keyword_set(xticklen) then xticklen=-0.02
if not keyword_set(yticklen) then yticklen=-0.02
if not keyword_set(xminor) then xminor=-1
if not keyword_set(yminor) then yminor=-1
if not keyword_set(mincol) then mincol=0
if not keyword_set(pos) then begin
    if !d.name eq 'X' then begin
        if not keyword_set(dxs) then begin
            pos=[70,70,400,400]
        endif else begin
            xs1=floor(dxs*70./500.)
            xs2=floor(dxs*400./500.)
            pos=[xs1,xs1,xs2,xs2]
        endelse
    endif else begin
        pos=[0.1,0.1,0.8,0.8]
    endelse
endif
if not keyword_set(plottau) then plottau=0
if plottau eq 0 then begin
    image=a.image
endif else begin
    image=a.tau
endelse
immir=rotate(image,2)
;
x=(findgen(a.nx)/(a.nx-1.d0)-0.5d0)*a.sizepix_x*a.nx*distscale
y=(findgen(a.ny)/(a.ny-1.d0)-0.5d0)*a.sizepix_y*a.ny*distscale
if !d.name eq 'X' then begin
    erase
    if not keyword_set(noimage) then begin
        z=congrid(immir,pos(2),pos(3))
        if keyword_set(saturate) then begin
	    dum = saturate*max(z)
	    z   = (z lt dum)*z + (z ge dum)*dum
	endif
        if keyword_set(thres) then begin
	    dum = thres*max(z)
	    z   = (z gt dum)*z - dum
        endif
        z=z/max(z)
        tv,bytscl(z,min=mincol,top=!d.table_size),pos(0),pos(1)
    endif
    plot,x,y,/noerase,position=[pos(0),pos(1),$
      pos(0)+pos(2),pos(1)+pos(3)],/device,/nodata,$
      xminor=xminor,yminor=yminor,xstyle=1,ystyle=1,$
      xticklen=xticklen,yticklen=yticklen,_EXTRA=_extra
    if keyword_set(contour) then begin
      contour,immir,x,y,nlevels=nlevels,position=[pos(0),pos(1),$
      pos(0)+pos(2),pos(1)+pos(3)],/device,$
      xminor=xminor,yminor=yminor,xstyle=1,ystyle=1,$
      xticklen=xticklen,yticklen=yticklen,/noerase,_EXTRA=_extra
    endif
endif else begin
    if not keyword_set(noimage) then begin
        z = immir
        if keyword_set(saturate) then begin
	    dum = saturate*max(z)
	    z   = (z lt dum)*z + (z ge dum)*dum
	endif
        if keyword_set(thres) then begin
	    dum = thres*max(z)
	    z   = (z gt dum)*z - dum
        endif
        z=z/max(z) 
        tv,bytscl(z,min=0.d0,top=!d.table_size),pos(0),pos(1),$
             xsize=pos(2),ysize=pos(3),/norm
    endif
    plot,x,y,/noerase,position=[pos(0),pos(1),$
      pos(0)+pos(2),pos(1)+pos(3)],/norm,/nodata,$
      xminor=xminor,yminor=yminor,xstyle=1,ystyle=1,$
      xticklen=xticklen,yticklen=yticklen,_EXTRA=_extra
    if keyword_set(contour) then begin
      contour,immir,x,y,nlevels=nlevels,position=[pos(0),pos(1),$
      pos(0)+pos(2),pos(1)+pos(3)],/norm,$
      xminor=xminor,yminor=yminor,xstyle=1,ystyle=1,$
      xticklen=xticklen,yticklen=yticklen,/noerase,_EXTRA=_extra
    endif
endelse
end




;----------------------------------------------------------
; WRITE IMAGE TO FILE IDL.PS
;----------------------------------------------------------
pro vi_write_image,state
print,'Writing idl.ps image...'
image=vi_read_recimage()
;
; First find out some more settings
;
if state.improcess_widgets ne 0 then begin
    WIDGET_CONTROL,state.wsl_thrs,get_value=txt
    val=float(txt)
    plot_thres=val(0)
    WIDGET_CONTROL,state.wsl_satur,get_value=txt
    val=float(txt)
    plot_saturate=val(0)
endif
if state.contour_widgets ne 0 then begin
    WIDGET_CONTROL,state.wbg_cont,get_value=txt
    txt=txt(0)
    plot_imcontr = txt + 0
;    WIDGET_CONTROL,state.wbg_invv,get_value=txt
;    txt=txt(0)
;    plot_invv = txt + 0
    plot_invv = 0
    WIDGET_CONTROL,state.wbg_lilg,get_value=txt
    txt=txt(0)
    plot_log = txt + 0
    WIDGET_CONTROL,state.wtx_nrc,get_value=txt
    txt=strcompress(string(txt(0)),/remove_all)
    txt=txt(0)
    plot_nrc = txt + 0
endif else begin
    plot_imcontr = 0
    plot_log = 0
    plot_nrc = 10
    plot_invv = 0
endelse
;
; If log, then do log
;
if plot_log eq 0 then begin
    imagedata = image
endif else begin
    imagedata = image
    ;;
    ;; Log this image, but make sure to have a limited range
    ;;
    imagedata.image = alog10(imagedata.image+1.d-200)
    mx              = max(imagedata.image)
    mn              = min(imagedata.image)
    if state.maxlog gt 0.d0 then begin
       if mx-mn gt state.maxlog then begin
          mn = mx - state.maxlog
       endif
    endif
    imagedata.image = imagedata.image - mn 
    i               = where(imagedata.image lt 0.d0)
    if i[0] ge 0 then $
    imagedata.image[i] = 0.d0
    ;;
    ;; Do the same for the tau
    ;;
    imagedata.tau   = alog10(imagedata.tau+1.d-200)
    mx              = max(imagedata.tau)
    mn              = min(imagedata.tau)
    if state.maxlog gt 0.d0 then begin
       if mx-mn gt state.maxlog then begin
          mn = mx - state.maxlog
       endif
    endif
    imagedata.tau   = imagedata.tau - mn
    i               = where(imagedata.tau lt 0.d0)
    if i[0] ge 0 then $
    imagedata.tau[i] = 0.d0
endelse
;
; If inverse video, then invert
;
if plot_invv ne 0 then begin
    imagedata.image = max(imagedata.image) - imagedata.image
    imagedata.tau = max(imagedata.tau) - imagedata.tau
endif
;
; Now plot things
;
set_plot,'ps',/interpolate
device,file='idl.ps',/portrait,/color,bits_per_pixel=24,font_size=fontsize,$
   xsize=16.0,ysize=16.0,yoffset=1
if plot_imcontr eq 0 then begin
    ;
    ; If image, then do image
    ;
    if state.improcess_widgets eq 1 then begin
        vi_plot_recimage,imagedata,thres=plot_thres,saturate=plot_saturate,$
                  plottau=state.plottau,pc=state.pc,au=state.au,$
                  dxs=state.draw_xsize
    endif else begin
        vi_plot_recimage,imagedata,$
                  plottau=state.plottau,pc=state.pc,au=state.au,$
                  dxs=state.draw_xsize
    endelse
endif else begin
    ;
    ; If contour, then contour
    ;
    vi_plot_recimage,imagedata,/contour,nlevels=plot_nrc,/noimage,$
                  plottau=state.plottau,pc=state.pc,au=state.au,$
                  dxs=state.draw_xsize
endelse
device,/close
set_plot,'x'
print,'..done'
end




;----------------------------------------------------------
; CALL PLOT IMAGE IN WIDGET BITMAP
;----------------------------------------------------------
pro plot_image_to_widget,state,image
;
; First find out some more settings
;
if state.improcess_widgets ne 0 then begin
    WIDGET_CONTROL,state.wsl_thrs,get_value=txt
    val=float(txt)
    plot_thres=val(0)
    WIDGET_CONTROL,state.wsl_satur,get_value=txt
    val=float(txt)
    plot_saturate=val(0)
endif
if state.contour_widgets ne 0 then begin
    WIDGET_CONTROL,state.wbg_cont,get_value=txt
    txt=txt(0)
    plot_imcontr = txt + 0
;    WIDGET_CONTROL,state.wbg_invv,get_value=txt
;    txt=txt(0)
;    plot_invv = txt + 0
    plot_invv = 0
    WIDGET_CONTROL,state.wbg_lilg,get_value=txt
    txt=txt(0)
    plot_log = txt + 0
    WIDGET_CONTROL,state.wtx_nrc,get_value=txt
    txt=strcompress(string(txt(0)),/remove_all)
    txt=txt(0)
    plot_nrc = txt + 0
endif else begin
    plot_imcontr = 0
    plot_log = 0
    plot_nrc = 10
    plot_invv = 0
endelse
if state.orient eq 0 then begin
   WIDGET_CONTROL,state.wtx_ango,get_value=txt
   txt=strcompress(string(txt(0)),/remove_all)
   state.maxlog = txt+0.e0
endif
;
; If log, then do log
;
if plot_log eq 0 then begin
    imagedata = image
endif else begin
    imagedata = image
    ;;
    ;; Log this image, but make sure to have a limited range
    ;;
    imagedata.image = alog10(imagedata.image+1.d-200)
    mx              = max(imagedata.image)
    mn              = min(imagedata.image)
    if state.maxlog gt 0.d0 then begin
       if mx-mn gt state.maxlog then begin
          mn = mx - state.maxlog
       endif
    endif
    imagedata.image = imagedata.image - mn 
    i               = where(imagedata.image lt 0.d0)
    if n_elements(i) gt 1 and i[0] ge 0 then imagedata.image[i] = 0.d0
    ;;
    ;; Do the same for the tau
    ;;
    imagedata.tau   = alog10(imagedata.tau+1.d-200)
    mx              = max(imagedata.tau)
    mn              = min(imagedata.tau)
    if state.maxlog gt 0.d0 then begin
       if mx-mn gt state.maxlog then begin
          mn = mx - state.maxlog
       endif
    endif
    imagedata.tau   = imagedata.tau - mn
    i               = where(imagedata.tau lt 0.d0)
    if n_elements(i) gt 1 and i[0] ge 0 then imagedata.tau[i] = 0.d0
endelse
;
; If inverse video, then invert
;
if plot_invv ne 0 then begin
    imagedata.image = max(imagedata.image) - imagedata.image
    imagedata.tau = max(imagedata.tau) - imagedata.tau
endif
;
; Now put things on the draw widget
;
wset,state.bitmap
if plot_imcontr eq 0 then begin
    ;
    ; If image, then do image
    ;
    if state.improcess_widgets eq 1 then begin
        vi_plot_recimage,imagedata,thres=plot_thres,saturate=plot_saturate,$
                  plottau=state.plottau,pc=state.pc,au=state.au,$
                  dxs=state.draw_xsize
    endif else begin
        vi_plot_recimage,imagedata,$
                  plottau=state.plottau,pc=state.pc,au=state.au,$
                  dxs=state.draw_xsize
    endelse
endif else begin
    ;
    ; If contour, then contour
    ;
    vi_plot_recimage,imagedata,/contour,nlevels=plot_nrc,/noimage,$
                  plottau=state.plottau,pc=state.pc,au=state.au,$
                  dxs=state.draw_xsize
endelse
wset,state.window
device,copy=[0,0,state.draw_xsize,state.draw_ysize,0,0,state.bitmap]
end




;----------------------------------------------------------
; RETURN THE FREQUENCY IN THE REQUESTED UNITS
;
;  0 = Hz
;  1 = ev
;  2 = kev
;----------------------------------------------------------
function vi_freq_in_units,unit_type,freq_hz
case unit_type of
    0 : freq = freq_hz
    1 : freq = 4.13568842841d-15 * freq_hz
    2 : freq = 4.13568842841d-18 * freq_hz
   -1 : freq = 2.9979d14 / freq_hz
endcase
return,freq
end



;----------------------------------------------------------
; RENDER THE IMAGE AND VIEW IT ON SCREEN
;----------------------------------------------------------
pro view_the_image,state
;
; Get the current values of the widgets
;
WIDGET_CONTROL,state.wtx_nx,get_value=txt
txt=strcompress(string(txt(0)),/remove_all)
nx = txt + 0
;WIDGET_CONTROL,state.wtx_ny,get_value=txt
;txt=strcompress(string(txt(0)),/remove_all)
;ny = txt + 0
ny=nx
WIDGET_CONTROL,state.wtx_sx,get_value=txt
txt=strcompress(string(txt(0)),/remove_all)
sx = txt + 0.e0
;WIDGET_CONTROL,state.wtx_sy,get_value=txt
;txt=strcompress(string(txt(0)),/remove_all)
;sy = txt + 0.e0
sy=sx
;
; Now the middle widgets
;
;bg_onemic = -1.d0
;bg_plaw   = 2.d0
ofsx      = 0.d0
ofsy      = 0.d0
WIDGET_CONTROL,state.wtx_ofsx,get_value=txt
txt=strcompress(string(txt(0)),/remove_all)
ofsx = txt + 0.e0
WIDGET_CONTROL,state.wtx_ofsy,get_value=txt
txt=strcompress(string(txt(0)),/remove_all)
ofsy = txt + 0.e0
if keyword_set(state.background) then begin
    WIDGET_CONTROL,state.wtx_bgtmp,get_value=txt
    txt=strcompress(string(txt(0)),/remove_all)
    state.bgtmp = txt + 0.e0
    WIDGET_CONTROL,state.wtx_bgdil,get_value=txt
    txt=strcompress(string(txt(0)),/remove_all)
    state.bgdil = txt + 0.e0
endif
WIDGET_CONTROL,state.wtx_angi,get_value=txt
txt=strcompress(string(txt(0)),/remove_all)
angi = ( txt + 0.e0 ) / 57.29
WIDGET_CONTROL,state.wtx_ango,get_value=txt
txt=strcompress(string(txt(0)),/remove_all)
if state.orient eq 1 then begin
   ango = ( txt + 0.e0 ) / 57.29
endif else begin
   ango = 0.e0
   state.maxlog = txt+0.e0
endelse
if state.nfr gt 1 then begin
    WIDGET_CONTROL,state.wsl_freq,get_value=txt
    txt=strcompress(string(txt(0)),/remove_all)
    ifreq = txt + 0
endif else begin
    ifreq = -1
 endelse
star=state.star
;
; Convert sx and sy
;
if state.pc ne 0 then begin
   sx   = sx * 3.08572d18
   sy   = sy * 3.08572d18
   ofsx = ofsx * 3.08572d18
   ofsy = ofsy * 3.08572d18
   if state.au ne 0 then stop    ; Not both au and pc possible
endif
if state.au ne 0 then begin
   sx   = sx * 1.496d13
   sy   = sy * 1.496d13
   ofsx = ofsx * 1.496d13
   ofsy = ofsy * 1.496d13
endif
;
; Call Radical to compute the image
;
if not keyword_set(state.background) then begin
    vi_radical_do_image,angi,nx,ny,sx,sy,ofsx,ofsy,ango,ifreq,star,$
        code=state.code
endif else begin
    vi_radical_do_image,angi,nx,ny,sx,sy,ofsx,ofsy,ango,ifreq,star,$
        /background,bgtmp=state.bgtmp,bgdil=state.bgdil,$
        code=state.code
                   ;;bgom=bg_onemic,bgpl=bg_plaw
endelse
;
; Read the image
;
imagedata=vi_read_recimage()
;
; Remember current window
;
;if defined(!d) then begin
    oldwin=1
    wincur=!d.window 
;endif else oldwin=0
;
; Now plot the image
;
plot_image_to_widget,state,imagedata
;;;;wset,state.bitmap
;;;;if state.improcess_widgets eq 1 then begin
;;;;    vi_plot_recimage,imagedata,thres=state.thres,saturate=state.saturate,$
;;;;                  plottau=state.plottau,pc=state.pc,au=state.au
;;;;endif else begin
;;;;    vi_plot_recimage,imagedata,$
;;;;                  plottau=state.plottau,pc=state.pc,au=state.au
;;;;endelse
;;;;wset,state.window
;;;;device,copy=[0,0,state.draw_xsize,state.draw_ysize,0,0,state.bitmap]
;
; Reset old window
;
if oldwin ne 0 then wset,wincur
end



;----------------------------------------------------------
; IMAGE VIEWER
;----------------------------------------------------------
pro viewimage,state=state,ifr=ifr,frunits=frunits,nobd=nobd,$
              expert=expert,tau=tau,star=star,maxlog=maxlog,$
              orient=orient,noimproc=noimproc,pc=pc,au=au,$
              small=small,bg=bg,hori=hori,background=background,$
              code=code
;
; Do a check
;
;; filename1 = "nonlte.dat"
;; str1=findfile(filename1)
;; filename2 = "dusttemp.inp"
;; str2=findfile(filename2)
;; if (str1(0) ne filename1) and (str2(0) ne filename2) then begin
;;     print,"Sorry, but there is no nonlte.dat or dusttemp.inp file"
;;     print,"This means that the iteration has not yet been done"
;;     print,"So, first do radical iterate to find solve the transfer"
;;     stop
;; endif
;
; Check radical.inp
;
qqq=findfile('radical.inp_bkviewimage',count=count)
if count gt 0 then begin        
    print,'WARNING: There exists a file radical.inp_bkviewimage'
    print,'         Presumably something went wrong in an old'
    print,'         run of viewimage with /background set on.'
    print,'    Solution: mv this file to radical.inp. But check' 
    print,'              if radical.inp_bkviewimage is an uncorrupted'
    print,'              one.'
    stop
endif
;
; Max values
;
im_nx_max        = 400
im_ny_max        = 400
;
; Default values
;
;if not keyword_set(code) then code = 1
if not keyword_set(code) then begin
   print,'Note: From now on RAYTRACE is used per default instead of RADICAL'
   code = 2
endif
if not keyword_set(small) then begin
    draw_xsize       = 500
    draw_ysize       = 500
endif else begin
    draw_xsize       = 400
    draw_ysize       = 400
endelse
xstxt            = 50
fsize_default    = 1
im_nx_default    = 100
im_ny_default    = 100
im_xsize_default = 1e17
im_ysize_default = 1e17
im_xoff_default  = 0e0
im_yoff_default  = 0e0
im_incl_default  = 80.e0
im_orient_default= 0e0
im_maxlog_default= 10.d0
if keyword_set(background) then begin
    bg_temp_default  = 3.d0
    bg_dil_default   = 1.d0
endif else begin
    bg_temp_default  = 0.d0
    bg_dil_default   = 0.d0
    background       = 0
    wtx_bgtmp        = 0
    wtx_bgdil        = 0
endelse
inu_default      = 1
str=findfile("dustopac.inp")
if(str(0) eq "dustopac.inp") then begin
    fr_units_default = -1
endif else begin 
    fr_units_default = 1
endelse
nrc_default      = 10
plot_log_default = 0
imcontr_default  = 0
improcess_widgets= 0
contour_widgets  = 0
if keyword_set(tau) then plottau=1 else plottau = 0
if keyword_set(frunits) then fr_units_default = frunits
if not keyword_set(star) then star=0
;;;if keyword_set(expert) then improcess_widgets = 1
;;;if keyword_set(expert) then contour_widgets = 1
improcess_widgets = 1
contour_widgets = 1
if keyword_set(maxlog) then im_maxlog_default=maxlog
if not keyword_set(orient) then orient=0
if keyword_set(noimproc) then improcess_widgets = 0
if not keyword_set(au) then au=0
if not keyword_set(pc) then pc=0
if not keyword_set(bg) then bg=0
;bg_onemic = 0.d0
;bg_plaw   = 2.0
;
; Find frequency file and read it
;
nfr = 0
z=findfile('frequency.inp',count=count)
if(count gt 0) then begin
    openr,1,'frequency.inp'
    readf,1,nfr
    frequency = fltarr(nfr)
    readf,1,frequency
    close,1
endif else begin
    nfr       = -1
    frequency = 0.d0
endelse
;
; Find radius file and read it
;
nr = 0
z=findfile('radius.inp',count=count)
if(count gt 0) then begin
    openr,1,'radius.inp'
    readf,1,nr
    radius = fltarr(nr)
    readf,1,radius
    im_xsize_default = radius(nr-1)
    im_ysize_default = radius(nr-1)
    close,1
endif else begin
    nr     = -1
    radius = 0.d0
endelse
if keyword_set(pc) then begin
    im_xsize_default = im_xsize_default / 3.08572d18
    im_ysize_default = im_ysize_default / 3.08572d18
    if keyword_set(au) then stop ; Not both pc and au allowed
endif
if keyword_set(au) then begin
    im_xsize_default = im_xsize_default / 1.496d13
    im_ysize_default = im_ysize_default / 1.496d13
endif
;
; Create the backdrop pixmap
;
if not keyword_set(nobd) then begin
   nobd=0
   window,/free,/pixmap,xsize=draw_xsize,ysize=draw_ysize
   bitmap=!d.window
endif else begin
   bitmap=0
   nobd=1
endelse
;
; The array for the image
;
image=fltarr(im_nx_max,im_ny_max)
;
;
; Creating the widgets
;
if keyword_set(hori) then begin
    wbase      = WIDGET_BASE(column=2)
endif else begin
    wbase      = WIDGET_BASE(column=1)
endelse
wbase1     = WIDGET_BASE(wbase,column=1)
wbase2     = WIDGET_BASE(wbase,column=1)
bbase1     = WIDGET_BASE(wbase1,column=2)
but_quit   = WIDGET_BUTTON(bbase1,value='Quit Viewer',$
             xsize=255,uvalue='but_quit')            
if plottau eq 0 then strrendim="Write Image" else $
         strrendim="Write Tau Image"
but_wrtim  = WIDGET_BUTTON(bbase1,value=strrendim,$ 
             xsize=255,uvalue='but_wrtim') 
wdr_main   = WIDGET_DRAW(wbase1,xsize=draw_xsize,$
             ysize=draw_ysize,frame=10)
;
; Create expert option widgets (with the image processing)
;
if improcess_widgets eq 1 then begin
  imbase1  = WIDGET_BASE(wbase1,column=1)
  ibase1   = WIDGET_BASE(imbase1,column=3,frame=1)
  wsl_thrs = CW_FSLIDER(ibase1,xsize=245,$
             minimum=0.d0,maximum=1.d0,$
             uvalue='wsl_thrs',value=0.d0,$
             /suppress_value)
  wsl_satur= CW_FSLIDER(ibase1,xsize=245,$
             minimum=0.d0,maximum=1.d0,$
             uvalue='wsl_satur',value=1.d0,$
             /suppress_value)
endif else begin
  imbase1  = 0
  ibase1   = 0
  wsl_thrs = 0
  wsl_satur= 0
endelse
;
; Create widgets for choice between contours and image
;
if contour_widgets eq 1 then begin
  imbase2  = WIDGET_BASE(wbase2,column=1)
  ibase2   = WIDGET_BASE(imbase2,column=7,frame=1)
  wbg_lilg = CW_BGROUP(ibase2,['lin','log'],$
             column=2,set_value=0,uvalue='wbg_lilg',$
             button_uvalue=['wbg_lin','wbg_log'],$
             /exclusive)
  wlb_ex1  = WIDGET_LABEL(ibase2,xsize=15,$
             ysize=30,uvalue='wlb_ex1',$
             value='  ')
;  wbg_invv = CW_BGROUP(ibase2,['norm','inv'],$
;             column=2,set_value=0,uvalue='wbg_invv',$
;             button_uvalue=['wbg_nrm','wbg_inv'],$
;             /exclusive)
  wbg_star = CW_BGROUP(ibase2,['star'],$
             column=1,uvalue='wbg_star',/nonexclusive)
  wlb_nrc  = WIDGET_LABEL(ibase2,xsize=80,$
             ysize=30,uvalue='wlb_nrc',$
             value='nlevels:')
  wtx_nrc  = WIDGET_TEXT(ibase2,xsize=6,$    
             ysize=1,frame=0,uvalue='wtx_nrc',$
             /editable)
  wlb_ex2  = WIDGET_LABEL(ibase2,xsize=12,$
             ysize=30,uvalue='wlb_ex2',$
             value='  ')
  wbg_cont = CW_BGROUP(ibase2,['image','contour'],$
             column=2,set_value=0,uvalue='wbg_cont',$
             button_uvalue=['wbg_img','wbg_cntr'],$
             /exclusive)
endif else begin
  imbase2  = 0
  ibase2   = 0
  wbg_lilg = 0
  wbg_cont = 0
  wtx_nrc  = 0
  wbg_star = 0
endelse
;
; Create positioning control widgets
;
tmbase1    = WIDGET_BASE(wbase2,column=3)
tbase1     = WIDGET_BASE(tmbase1,column=2,frame=1)
wlb_nx     = WIDGET_LABEL(tbase1,xsize=xstxt,$
             ysize=30,uvalue='wlb_nx',$
             value='N_x')
wlb_sx     = WIDGET_LABEL(tbase1,xsize=xstxt,$
             ysize=30,uvalue='wlb_sx',$
             value='Size X')
wtx_nx     = WIDGET_TEXT(tbase1,xsize=10,$    
             ysize=1,frame=0,uvalue='wtx_nx',$
             /editable)                       
wtx_sx     = WIDGET_TEXT(tbase1,xsize=10,$    
             ysize=1,frame=0,uvalue='wtx_sx',$
             /editable)                       
wtx_sy=0
wtx_ny=0
if bg eq 0 then begin
    ;;
    ;; For the offset
    ;;
    tbase3     = WIDGET_BASE(tmbase1,column=2,frame=1)
    wlb_ofsx   = WIDGET_LABEL(tbase3,xsize=xstxt,$
                 ysize=30,uvalue='wlb_ofsx',$
                 value='Offset X')
    wlb_ofsy   = WIDGET_LABEL(tbase3,xsize=xstxt,$
                 ysize=30,uvalue='wlb_ofsy',$
                 value='Offset Y')
    wtx_ofsx   = WIDGET_TEXT(tbase3,xsize=10,$    
                 ysize=1,frame=0,uvalue='wtx_ofsx',$
                 /editable)                       
    wtx_ofsy   = WIDGET_TEXT(tbase3,xsize=10,$    
                 ysize=1,frame=0,uvalue='wtx_ofsy',$
                 /editable)                       
endif else begin
    ;;
    ;; Instead of offset, widgets for the background illumination
    ;;
    tbase3     = WIDGET_BASE(tmbase1,column=2,frame=1)
    wlb_ofsx   = WIDGET_LABEL(tbase3,xsize=80,$
                 ysize=30,uvalue='wlb_ofsx',$
                 value='Bg @ 1mic')
    wlb_ofsy   = WIDGET_LABEL(tbase3,xsize=80,$
                 ysize=30,uvalue='wlb_ofsy',$
                 value='Bg Plaw')
    wtx_ofsx   = WIDGET_TEXT(tbase3,xsize=10,$    
                 ysize=1,frame=0,uvalue='wtx_ofsx',$
                 /editable)                       
    wtx_ofsy   = WIDGET_TEXT(tbase3,xsize=10,$    
                 ysize=1,frame=0,uvalue='wtx_ofsy',$
                 /editable)
endelse
tbase4     = WIDGET_BASE(tmbase1,column=2,frame=1)
wlb_angi   = WIDGET_LABEL(tbase4,xsize=90,$
             ysize=30,uvalue='wlb_angi',$
             value='Inclination')
if orient eq 1 then begin
   wlb_ango   = WIDGET_LABEL(tbase4,xsize=xstxt,$
             ysize=30,uvalue='wlb_angp',$
             value='Orientation')
endif else begin
   wlb_ango   = WIDGET_LABEL(tbase4,xsize=xstxt,$
             ysize=30,uvalue='wlb_angp',$
             value='Max Log10')
endelse
wtx_angi   = WIDGET_TEXT(tbase4,xsize=10,$    
             ysize=1,frame=0,uvalue='wtx_angi',$
             /editable)
wtx_ango   = WIDGET_TEXT(tbase4,xsize=10,$    
             ysize=1,frame=0,uvalue='wtx_angp',$
             /editable)                       
if keyword_set(background) then begin
    tmbase3    = WIDGET_BASE(wbase2,frame=1,row=1)
    wlb_bgtmp  = WIDGET_LABEL(tmbase3,xsize=90,$
             ysize=30,uvalue='wlb_bgtmp',$
             value='BG Temp')
    wtx_bgtmp  = WIDGET_TEXT(tmbase3,xsize=10,$    
             ysize=1,frame=0,uvalue='wtx_bgtmp',$
             /editable)
    wlb_bgdil  = WIDGET_LABEL(tmbase3,xsize=90,$
             ysize=30,uvalue='wlb_bgdil',$
             value='BG Dilution')
    wtx_bgdil  = WIDGET_TEXT(tmbase3,xsize=10,$    
             ysize=1,frame=0,uvalue='wtx_bgdil',$
             /editable)    
endif
tmbase2    = WIDGET_BASE(wbase2,row=1)
xsize=520
if plottau eq 0 then strrendim="Render Image" else $
         strrendim="Render Tau Image"
if nfr gt 1 then begin
    but_makeim = WIDGET_BUTTON(tmbase2,value=strrendim,$
                 xsize=0.4*xsize,ysize=35,uvalue='but_makeim')
    wsl_freq   = WIDGET_SLIDER(tmbase2,minimum=1,maximum=nfr,$  
                 xsize=0.3*xsize,uvalue='wsl_freq',value=1,/drag)
    wtx_freq   = WIDGET_TEXT(tmbase2,xsize=14,$
                 ysize=1,uvalue='wtx_freq')   
endif else begin
    but_makeim = WIDGET_BUTTON(tmbase2,value=strrendim,$
             xsize=xsize,ysize=35,uvalue='but_makeim')
    wsl_freq   = 0
    wtx_freq   = 0
endelse
;
; Put things on the screen
;
WIDGET_CONTROL,wbase,/realize
WIDGET_CONTROL,wdr_main,get_value=window
;
; Now make a big structure with all info
;
state={window:window,wdr_main:wdr_main,wtx_sx:wtx_sx,wtx_sy:wtx_sy,$
       wtx_nx:wtx_nx,wtx_ny:wtx_ny,wtx_ofsx:wtx_ofsx,wtx_ofsy:wtx_ofsy,$
       wtx_angi:wtx_angi,wtx_ango:wtx_ango,wsl_freq:wsl_freq,$
       wtx_freq:wtx_freq,fr_units:fr_units_default,nfr:nfr,$
       wbg_lilg:wbg_lilg,wbg_cont:wbg_cont,wtx_nrc:wtx_nrc,wbg_star:wbg_star,$
       wsl_thrs:wsl_thrs,wsl_satur:wsl_satur,orient:orient,$
       frequency:frequency,bitmap:bitmap,nobd:nobd,maxlog:im_maxlog_default,$
       improcess_widgets:improcess_widgets,$
       contour_widgets:contour_widgets,draw_xsize:draw_xsize,$
       draw_ysize:draw_ysize,plottau:plottau,star:star,pc:pc,au:au,$
       background:background,bgtmp:bg_temp_default,bgdil:bg_dil_default,$
       wtx_bgtmp:wtx_bgtmp,wtx_bgdil:wtx_bgdil,code:code}

;;bg:bg,bg_onemic:bg_onemic,bg_plaw:bg_plaw}
;
; Give this structure to the main widget 
;
WIDGET_CONTROL,wbase,set_uvalue=state
;
; Fill text widgets with the text
;
case state.fr_units of
    0: strunit='Hz'
    1: strunit='eV'
    2: strunit='KeV'
   -1: strunit='mum'
endcase
;
WIDGET_CONTROL,wtx_nx,set_value=string(format='(I4)',im_nx_default)
WIDGET_CONTROL,wtx_sx,set_value=string(format='(E9.2)',im_xsize_default)
;if bg eq 0 then begin
WIDGET_CONTROL,wtx_ofsx,set_value=string(format='(E9.2)',im_xoff_default)
WIDGET_CONTROL,wtx_ofsy,set_value=string(format='(E9.2)',im_yoff_default)
;endif else begin
;    WIDGET_CONTROL,wtx_ofsx,set_value=string(format='(E9.2)',bg_onemic)
;    WIDGET_CONTROL,wtx_ofsy,set_value=string(format='(E9.2)',bg_plaw)
;endelse
WIDGET_CONTROL,wtx_angi,set_value=string(format='(F9.2)',im_incl_default)
if orient eq 1 then begin
   WIDGET_CONTROL,wtx_ango,set_value=string(format='(F9.2)',im_orient_default)
endif else begin
   WIDGET_CONTROL,wtx_ango,set_value=string(format='(F9.2)',im_maxlog_default)
endelse
if wtx_freq ne 0 then begin
    WIDGET_CONTROL,wtx_freq,set_value=string(format='(E9.2)',vi_freq_in_units($
     state.fr_units,frequency(0)))+'  '+strunit
endif
if wtx_nrc ne 0 then begin
    WIDGET_CONTROL,wtx_nrc,set_value=string(format='(I4)',nrc_default)
endif
if keyword_set(background) then begin
    WIDGET_CONTROL,wtx_bgtmp,set_value=string(format='(F9.2)',bg_temp_default)
    WIDGET_CONTROL,wtx_bgdil,set_value=string(format='(F9.2)',bg_dil_default)
endif
;WIDGET_CONTROL,wtx_ny,set_value=string(format='(I4)',im_ny_default)
;WIDGET_CONTROL,wtx_sy,set_value=string(format='(E9.2)',im_ysize_default)
;
;;if wtx_freq ne 0 then begin
;;    WIDGET_CONTROL,wtx_freq,set_value=string(format='(E9.2)',$
;;      freq_in_units(state.fr_units,state.physvar.grid.frequency(state.ifreq)))+$
;;      '  '+strunit
;;endif
;;widgets_params_fill,state
;
; Make a first plot
;
view_the_image,state
;
; Now activate the widget
;
xmanager,'viewimage',wbase,/NO_BLOCK
end


;----------------------------------------------------------
; WIDGET EVENT HANDLER
;----------------------------------------------------------
PRO viewimage_event,sevent
WIDGET_CONTROL,sevent.id,get_uvalue=uvalue
WIDGET_CONTROL,sevent.top,get_uvalue=state,/no_copy
case uvalue of
    'but_quit': begin
        WIDGET_CONTROL,sevent.top,/destroy
        !except=1
        return
    end
    'but_makeim': begin
        view_the_image,state
    end
    'but_wrtim': begin
        vi_write_image,state
    end
    'wsl_freq': begin
        WIDGET_CONTROL,state.wsl_freq,get_value=txt
        txt=strcompress(string(txt(0)),/remove_all)
        ifreq = txt + 0
        case state.fr_units of
            0: strunit='Hz'
            1: strunit='eV'
            2: strunit='KeV'
            -1: strunit='mum'
        endcase
        if state.wtx_freq ne 0 then begin
            WIDGET_CONTROL,state.wtx_freq,set_value=$
                string(format='(E9.2)',vi_freq_in_units(state.fr_units,$
                state.frequency(ifreq-1)))+'  '+strunit
        endif
    end
    'wsl_thrs': begin
;        WIDGET_CONTROL,sevent.id,get_value=txt
;        val=float(txt)
;        state.thres=val(0)
	aa=vi_read_recimage()
        plot_image_to_widget,state,aa
    end
    'wsl_satur': begin
;        WIDGET_CONTROL,sevent.id,get_value=txt
;        val=float(txt)
;        state.saturate=val(0)
	aa=vi_read_recimage()
        plot_image_to_widget,state,aa
    end
    'wbg_lilg': begin
	aa=vi_read_recimage()
        plot_image_to_widget,state,aa	
    end
    'wtx_angp': begin
	aa=vi_read_recimage()
        plot_image_to_widget,state,aa	
    end
    'wbg_cont': begin
	aa=vi_read_recimage()
        plot_image_to_widget,state,aa	
    end
    'wtx_nrc': begin
	aa=vi_read_recimage()
        plot_image_to_widget,state,aa	
    end
    'wbg_star': begin
        state.star = 1-state.star
	aa=vi_read_recimage()
        plot_image_to_widget,state,aa	
    end
    else: notrecog=1
endcase
WIDGET_CONTROL,sevent.top,set_uvalue=state,/no_copy
end


