#define INPUT_MAX_LEN 100

#include "main.h"

      program raytrace
      implicit none
      integer inu,iformat,i,l
      logical fex,ierror
      doubleprecision inclination,dustmass,fraccov
      doubleprecision total_dust_mass
      character*160 string
c
#include "common_grid.h"
#include "common_main.h"
#include "common_dust.h"
#include "common_telescope.h"
c
c     Default stuff
c
      main_nrphiinf             = 32
      main_nrrayextra           = -20 
      telesc_dbdr               = 1
      inclination               = 45.
      main_anginf               = inclination * 0.01745329252d0
      main_imethod              = 1
      main_nrref                = 10
c      main_distance             = 3.08572d18
      main_distapert            = 3.08572d18
c
c     Write a banner:
c
      write(*,*) ' '
      write(*,*) '==================================================='
      write(*,*) '        RAY-TRACER FOR IMAGES AND SPECTRA          '
      write(*,*) '==================================================='
      write(*,*) ' '
c
c     Read the input information about the model setup (dust opacities,
c     radial and theta grid, the dust density distribution, the stellar
c     info).
c     
      call read_all()
c
c     Check if radical.inp is there, and if so, then warn
c
      inquire(file='radical.inp',exist=fex)
      if(fex) then
          write(*,*) 'WARNING: I find a file radical.inp. This is'
          write(*,*) '   for RADICAL used as a ray-tracer. With the'
          write(*,*) '   new RAYTRACE program you are now evidently'
          write(*,*) '   using you do not need an input file. You'
          write(*,*) '   can remove radical.inp. If you wish to'
          write(*,*) '   fine-tune RAYTRACE, then you can do so with'
          write(*,*) '   an input file raytrace.inp (which replaces'
          write(*,*) '   the functionality of radical.inp).'
      endif
c
c     Read the raytrace setting file (if it does not exist, then
c     the default values are taken)
c
      inquire(file='raytrace.inp',exist=fex)
      if(fex) then 
          open(unit=1,file='raytrace.inp')
          read(1,'(A80)') string
          close(1)
          l=len_trim(string)
          if(l.eq.0) then
              string='a'
              l=1
          endif
          i=0
 10       i=i+1
          if(i.ge.l) goto 15
          if(string(i:i).eq.' ') goto 10
 15       continue
          if((string(i:i).eq.'1').or.(string(i:i).eq.'2').or.
     %       (string(i:i).eq.'3').or.(string(i:i).eq.'4').or.
     %       (string(i:i).eq.'5').or.(string(i:i).eq.'6').or.
     %       (string(i:i).eq.'7').or.(string(i:i).eq.'8').or.
     %       (string(i:i).eq.'9')) then 
c
c             Old formatted style input 
c
c             ...Old defaults
c             
              main_imethod              = 0
c
c             ...Read
c
              open(unit=1,file='raytrace.inp')
              read(1,*) iformat
              if((iformat.ne.1).and.(iformat.ne.2)) then 
                  write(*,*) 'ERROR: Cannot read file format ',iformat,
     %                 ' in file raytrace.inp'
                  stop
              endif
              read(1,*) main_nrphiinf
              read(1,*) main_nrrayextra
              if(iformat.ge.2) then
                  read(1,*) main_imethod
                  if(main_imethod.ge.1) then 
                      read(1,*) main_nrref
                  endif
              endif
              read(1,*) telesc_dbdr
              read(1,*) inclination
              close(1)
              main_anginf = inclination * 0.01745329252d0
          else
c
c             New namelist style input
c
              open(unit=1,file='raytrace.inp')
              call read_input_file()
              close(1)
              call parse_input_integer('nrphiinf@',main_nrphiinf)
              call parse_input_integer('nrrayextra@',main_nrrayextra)
              call parse_input_integer('imethod@',main_imethod)
              call parse_input_integer('nrref@',main_nrref)
              call parse_input_integer('dbdr@',telesc_dbdr)
              call parse_input_double('inclination@',inclination)
              call check_all_lines_ok(ierror)
              if(ierror) stop
              main_anginf = inclination * 0.01745329252d0
          endif
      else
          write(*,*) 'No raytrace.inp found --> using default settings.'
      endif
c
c     Warning
c
      if(main_imethod.lt.1) then
          write(*,*) 'WARNING: Without extra refined rays just inward'
          write(*,*) '         of inner rim you *might* get problems'
          write(*,*) '         of missing NIR flux at near-face-on'
          write(*,*) '         inclinations. Advice: use imethod=1'
          write(*,*) '         and nrref=10 in raytrace.inp to avoid'
          write(*,*) '         this potential problem.'
      endif
c
c     Read the command file, to know what the user wants us to do
c
      call read_command_file()
c
c     If a spectrum is to be computed, and an aperture is requested,
c     then read the aperture file
c
      if(telesc_command.eq.2) then
          if(main_globapert.ne.0.d0) then
              do inu=1,freq_nr
                  spec_aperture(inu) = main_globapert
              enddo
              telesc_aperture = 1
          else
              call read_aperture_file()
          endif
      endif
c
c     Compute, for safety, the dust mass of the entire setup
c
      dustmass = total_dust_mass(-1,0)
      write(*,*) 'Total dust mass in setup: ',dustmass *
     %               5.02512562814d-34,' M_sun (cell-centered grid)'
      dustmass = total_dust_mass(-1,1)
      write(*,*) 'Total dust mass in setup: ',dustmass *
     %               5.02512562814d-34,' M_sun (corner-centered grid)'
c
c     Now arrange the pixels for the images/spectra
c
      if((telesc_command.eq.1).or.(telesc_command.eq.2).or.
     %   (telesc_command.eq.4).or.(telesc_command.eq.5).or.
     %   (telesc_command.eq.7)) then
c
c         This is for circular images or for spectra: arrange the
c         pixels in concentric circles around the star, with 
c         log-spaced radial separation, so that all scales are
c         well represented.
c
          call setup_rays_circular(1,irsi_frsizex,1,main_anginf,
     %              main_nrphiinf,main_nrrayextra,telesc_dbdr,
     %              radbnd_rstar,main_imethod,main_nrref)
c
      elseif((telesc_command.eq.3).or.(telesc_command.eq.6).or.
     %       (telesc_command.eq.8)) then
c
c         This is for rectangular (normal) images
c
          call setup_rays_rectang(imr_nx,imr_ny,imr_spx,imr_spy,
     %                  main_anginf,imr_phioff,imr_xoff,imr_yoff)
c
c      else
cc          write(*,*) 'INTERNAL ERROR. Warn author.'
cc          write(*,*) '  telesc_command unknown'
c           write(*,*) 'Nothing to do...'
c         stop
      endif
c
c     Now make the image or the corresponding spectrum
c
      if(telesc_command.eq.1) then
c
c         CIRCULAR IMAGE
c
          write(*,*) 'Rendering the image at distant observer'
          write(*,*) '   Inclination = ',
     %         main_anginf*57.2957795129d0,' degrees'
          call make_freq_image_circular(image_ifreq)
          write(*,*) 'Writing the image to file'
          call write_image_circular(image_ifreq)
      endif
      if(telesc_command.eq.2) then
c
c         SPECTRUM AT D = 1 PARSEC
c
          write(*,*) 'Rendering the spectrum at distant observer'
          write(*,*) '   Inclination = ',
     %         main_anginf*57.2957795129d0,' degrees'
          if(telesc_inostar.eq.1) then
              write(*,*) 'Star is excluded in spectrum'
          endif
          write(*,*) 'Computing spectrum'
c
c         MODIFICATION 09.07.07
c
ccc          if(main_distance.eq.0.d0) main_distance=3.08572d18
          call calc_spectrum(main_distapert,0,0,0,telesc_inostar)
          write(*,*) 'Writing the spectrum to file'
          call write_spectrum(-1)
      endif
      if(telesc_command.eq.3) then
c
c         RECTANGULAR IMAGE
c              
          write(*,*) 'Rendering the rectangular image'
          call make_freq_image_rectang(image_ifreq)
          write(*,*) 'Writing the image to file'
          call write_image_rectang(image_ifreq)
      endif
      if(telesc_command.eq.7) then
c
c         SEQUENCE OF CIRCULAR IMAGES
c              
          call create_and_write_circ_images(
     %              imseq_inu0,imseq_inu1)
      endif
      if(telesc_command.eq.8) then
c
c         SEQUENCE OF RECTANGULAR IMAGES
c              
          call create_and_write_rectang_images(
     %              imseq_inu0,imseq_inu1)
      endif
      if(telesc_command.eq.9) then
c
c         SPECTRUM AT D = 1 PARSEC FOR ALL INCLINATIONS
c         IN ORDER TO CHECK ENERGY CONSERVATION OR GET 
c         A FULL SERIES OF SPECTRA AT ALL INCLINATIONS
c
          call comp_covering_fraction(fraccov)
          write(*,*) 'Covering fraction     = ',
     %         fraccov
          call calc_spec_all_incl(telesc_nincl,telesc_writespec)
          if(telesc_inostar.eq.0) then
              write(*,*) 'L_tot / L_star (glob) = ',
     %             spec_lum/radbnd_lstar
              write(*,*) 'Error in reprocessing = ',
     %             abs(1.d0-spec_lum/radbnd_lstar)/fraccov
          else
              write(*,*) 'L_star / L_sun         = ',
     %             radbnd_lstar/3.85252737833d33
              write(*,*) 'L_dust / L_star (glob) = ',
     %             spec_lum/radbnd_lstar
              write(*,*) 'Error in reprocessing = ',
     %             abs((spec_lum/radbnd_lstar)/fraccov-1.d0)
          endif
      endif
c
c     Done with the program
c     
      end






c     --------------------------------------------------------------
c                       READ THE COMMAND FILE 
c     --------------------------------------------------------------
      subroutine read_command_file()
      implicit none
c
#include "common_grid.h"
#include "common_main.h"
#include "common_dust.h"
#include "common_telescope.h"
c
      integer idummy,iskyangle,unit
      character*80 string,wlstr,szunit
      doubleprecision lambda,frequency,wlfact,scalefact,distance
      doubleprecision freqmin,freqmax,apert
      logical fexists,strread
c
c     Open the command file
c
      open(unit=2,file='command',status='old',err=701)
      goto 702
  701 continue
      write(*,*) 'Error opening command file. Should exist, but ',
     %           'something goes wrong'
      stop 13
  702 continue
c     
c     Read the command
c     
      read(2,*) string
c
c     Now parse this thing
c
      if(string(1:5).eq.'image') then
c     
c         COMMAND: IMAGE
c     
c         So... the user wants to produce an image at infinity
c     
          telesc_command = 3
          scalefact = 1.d0
          iskyangle = 0
          distance = -1.d0
          strread = .false.
  761     continue
          if(.not.strread) then 
              read(2,*,end=763,err=763) string
          endif
          strread = .false.
          if(string(1:7).eq.'addstar') then
              imrec_addstar = 1
          endif
          if(string(1:5).eq.'angle') then
              write(*,*) '...reading angle'
              read(2,*,end=762,err=762) main_anginf
          endif
          if(string(1:4).eq.'incl') then
              write(*,*) '...reading inclination'
              read(2,*,end=762,err=762) main_anginf
              main_anginf = main_anginf * 1.745329252d-2
          endif
          if(string(1:4).eq.'npix') then
              read(2,*,end=762,err=762) imr_nx,imr_ny
              write(*,*) '...npix: ',imr_nx,imr_ny
          endif
          if(string(1:7).eq.'sizepix') then
              read(2,*,end=762,err=762) imr_spx,imr_spy
              write(*,*) '...size pix: ',imr_spx,imr_spy
          endif
          if(string(1:4).eq.'dist') then
              read(2,*,end=762,err=762) distance
              read(2,*,end=765,err=765) string
              if((string(1:2).eq.'AU').or.
     %            (string(1:2).eq.'au')) then
                  distance = 4.848139d-6 * distance
              elseif((string(1:2).eq.'PC').or.
     %               (string(1:2).eq.'pc').or.
     %               (string(1:6).eq.'parsec')) then
                  distance = 1.d0 * distance
              elseif(string(1:2).eq.'cm') then
                  distance = 3.2407347d-19 * distance
              else
                  strread = .true.
                  goto 761
              endif              
          endif
          if(string(1:9).eq.'imagesize') then
              read(2,*,end=762,err=762) imr_szimx,imr_szimy
              read(2,*,end=766,err=762) szunit
              goto 767
 766          continue
              write(*,*) 'Sorry, must specify size unit'
              stop 13
 767          continue
              if((szunit(1:2).eq.'AU').or.
     %            (szunit(1:2).eq.'au')) then
                  scalefact = 1.496d13
              elseif((szunit(1:2).eq.'PC').or.
     %               (szunit(1:2).eq.'pc').or.
     %               (szunit(1:6).eq.'parsec')) then
                  scalefact = 3.08572d18
              elseif(szunit(1:2).eq.'cm') then
                  scalefact = 1.d0
              elseif(szunit(1:6).eq.'arcsec') then
                  iskyangle = 1
                  scalefact = 1.496d13
              else
                  write(*,*) 'Sorry, do not know unit type ',szunit
                  stop 13
              endif
              write(*,*) '...size image: ',imr_szimx,imr_szimy
          endif
          if(string(1:6).eq.'offset') then
              read(2,*,end=762,err=762) imr_xoff,
     %                           imr_yoff,imr_phioff
              write(*,*) '...offsets: ',imr_xoff,
     %                           imr_yoff,imr_phioff
          endif
          if(string(1:5).eq.'ifreq') then
              read(2,*,end=762,err=762) image_ifreq
              write(*,*) '...ifreq: ',image_ifreq
          endif
          if((string(1:6).eq.'lambda').or.(string(1:5).eq.'lamda')
     %         .or.(string(1:5).eq.'labda')) then
              read(2,*,end=762,err=762) lambda
              read(2,*,end=763,err=763) string
              if(string(1:6).eq.'micron') then
                  wlfact = 2.9979d14
                  wlstr  = 'micron'
              else
                  write(*,*) 'Do not know wavelength unit ',string
                  stop 13
              endif
              freqmin = freq_nu(1)
              freqmax = freq_nu(freq_nr)
              if(freqmax.lt.freqmin) then
                  freqmin = freq_nu(freq_nr)
                  freqmax = freq_nu(1)
              endif
              frequency = wlfact / lambda
              if((frequency.lt.freqmin).or.
     %             (frequency.gt.freqmax)) then
                  write(*,*) 'Wavelength ',lambda,' ',wlstr,' out ',
     %                 'of range: [',wlfact/freqmax,
     %                 ',',wlfact/freqmin,'] ',wlstr
                  stop 13
              endif
              call hunt(freq_nu,freq_nr,frequency,image_ifreq)
              if((image_ifreq.lt.1).or.
     %             (image_ifreq.gt.freq_nr)) then
                  write(*,*) 'Weird bug in wavelength search...'
                  stop 77
              endif
              if(image_ifreq.ne.freq_nr) then
                  if((abs(frequency-freq_nu(image_ifreq))
     %                 .gt.5.d-2*freq_nu(image_ifreq)).and.
     %                 (abs(frequency-freq_nu(image_ifreq+1))
     %                 .gt.5.d-2*freq_nu(image_ifreq+1))) then
                      write(*,*) 'Warning: Taking nearest ',
     %                     'wavelength point near lambda=',
     %                     lambda,' ',wlstr
                      write(*,*) '   which is at ',
     %                     wlfact/freq_nu(image_ifreq),' ',wlstr
                  endif
              endif
              write(*,*) '...ifreq: ',image_ifreq
          endif
 765      continue
c
c         Now finish
c
          goto 761
  762     continue
          write(*,*) 'WARNING: Wanted to read data from the ',
     %        'command file, but somehow could not do it...'
          write(*,*) 'Using defaults'
  763     continue
c
c         Scale the image size
c
          if(iskyangle.eq.0) then
              imr_szimx = scalefact * imr_szimx
              imr_szimy = scalefact * imr_szimy
          else
              if(distance.gt.0.d0) then
                  imr_szimx = scalefact * distance * imr_szimx
                  imr_szimy = scalefact * distance * imr_szimy
              else
                  write(*,*) 'Sorry, when you specify ',szunit
                  write(*,*) 'you also must specify the ',
     %                        'distance in parsec...'
                  stop 13
              endif
          endif
c
c         Specify pixel size
c
          if(imr_szimx.gt.0.d0) then
              imr_spx = imr_szimx / (0.5d0*imr_nx)
          endif
          if(imr_szimy.gt.0.d0) then
              imr_spy = imr_szimy / (0.5d0*imr_ny)
          endif
      endif
      if(string(1:8).eq.'spectrum') then
c     
c         COMMAND: SPECTRUM
c     
          telesc_command = 2
          telesc_inostar = 0
          telesc_aperture = 0
 811      continue
          read(2,*,end=813) string
          if(string(1:5).eq.'angle') then
              read(2,*,end=812,err=812) main_anginf
          endif
          if(string(1:4).eq.'incl') then
              read(2,*,end=812,err=812) main_anginf
              main_anginf = main_anginf * 1.745329252d-2
          endif
          if(string(1:6).eq.'nostar') then
              telesc_inostar = 1
          endif
          if(string(1:5).eq.'apert') then
              telesc_aperture = 1
              read(2,*,end=812,err=812) apert
              read(2,*,end=811,err=811) string
              unit=0
              if((string(1:2).eq.'AU').or.
     %            (string(1:2).eq.'au')) then
                  unit=2
              elseif((string(1:2).eq.'PC').or.
     %               (string(1:2).eq.'pc').or.
     %               (string(1:6).eq.'parsec')) then
                  unit=3
              elseif((string(1:2).eq.'AS').or.
     %               (string(1:2).eq.'as').or.
     %               (string(1:6).eq.'arcsec')) then
                  unit=0
              elseif(string(1:2).eq.'cm') then
                  unit=1
              else
                  strread = .true.
                  goto 811
              endif              
          endif
c
c         MODIFICATION 09.07.07: introducing main_distapert instead
c            of main_distance.
c
          if(string(1:3).eq.'dpc') then
              read(2,*,end=812,err=812) main_distapert
              main_distapert = main_distapert * 3.08572d18
          endif
          goto 811
  812     continue
          write(*,*) 'WARNING: Wanted to read from the ',
     %        'command file, but somehow could not do it...'
  813     continue
          if(telesc_aperture.ne.0) then
              if(unit.eq.0) then          ! apert in arcsec
                  main_globapert = apert
              elseif(unit.eq.1) then      ! apert in centimeter
                  main_globapert = (apert/1.496d13)/
     %                             (main_distapert/3.08572d18)
              elseif(unit.eq.2) then      ! apert in AU
                  main_globapert = apert/
     %                             (main_distapert/3.08572d18)
              elseif(unit.eq.3) then      ! apert in pc
                  main_globapert = apert*2.06264705882d5/
     %                             (main_distapert/3.08572d18)
              else
                  stop 89210
              endif
          else
              main_globapert = 0.d0
          endif
      endif
      if(string(1:4).eq.'glob') then
c     
c         COMMAND: SPECTRUM AT ALL THETA
c     
          telesc_command = 9
 821      continue
          read(2,*,err=822,end=822) string
          if(string(1:5).eq.'nincl') then
              read(2,*,end=822,err=822) telesc_nincl
          endif
          if(string(1:5).eq.'write') then
              telesc_writespec = 1
          endif
          if(string(1:6).eq.'nostar') then
              telesc_inostar = 1
          endif
          goto 821
  822     continue
      endif
      if((string(1:11).eq.'cirimageseq').or.
     %   (string(1:5).eq.'circu')) then
c     
c         COMMAND: SEQUENCE OF CIRCULAR IMAGES
c     
          telesc_command = 7
          imseq_inu0 = 1
          imseq_inu1 = freq_nr
 914      continue
          read(2,*,end=913,err=913) string
          if(string(1:5).eq.'angle') then
              read(2,*,end=912,err=912) main_anginf
          endif
          if(string(1:4).eq.'incl') then
              read(2,*,end=912,err=912) main_anginf
              main_anginf = main_anginf * 1.745329252d-2
          endif
          if(string(1:4).eq.'inu0') then
              read(2,*,end=912,err=912) imseq_inu0
          endif
          if(string(1:4).eq.'inu1') then
              read(2,*,end=912,err=912) imseq_inu1
          endif
          goto 914
 912      continue
          write(*,*) 'WARNING: Wanted to read data from the ',
     %        'command file, but somehow could not do it...'
          write(*,*) 'Using defaults'          
 913      continue
          if((imseq_inu0.lt.1).or.(imseq_inu1.gt.freq_nr)) then
              write(*,*) 'Sorry, but in the command file you'
              write(*,*) '       specified inconsistent '
              write(*,*) '       inu0 or inu1'
              write(*,*) imseq_inu0,imseq_inu1,freq_nr
              stop 13
          endif
      endif
      if(string(1:11).eq.'recimageseq') then
c     
c         COMMAND: SEQUENCE OF RECTANGULAR IMAGES
c     
          telesc_command = 8
          imseq_inu0 = 1
          imseq_inu1 = freq_nr
          scalefact = 1.d0
          iskyangle = 0
          distance = -1.d0
          strread = .false.
 1001     continue
          if(.not.strread) then
              read(2,*,end=1013,err=1013) string
          endif
          strread = .false.
          if(string(1:7).eq.'addstar') then
              imrec_addstar = 1
          endif
          if(string(1:5).eq.'angle') then
              read(2,*,end=1012,err=1012) main_anginf
          endif
          if(string(1:4).eq.'incl') then
              read(2,*,end=1012,err=1012) main_anginf
              main_anginf = main_anginf * 1.745329252d-2
          endif
          if(string(1:4).eq.'inu0') then
              read(2,*,end=1012,err=1012) imseq_inu0
          endif
          if(string(1:4).eq.'inu1') then
              read(2,*,end=1012,err=1012) imseq_inu1
          endif
          if(string(1:4).eq.'npix') then
              read(2,*,end=1012,err=1012) imr_nx,imr_ny
c              write(*,*) '...npix: ',imr_nx,imr_ny
          endif
          if(string(1:7).eq.'sizepix') then
              read(2,*,end=1012,err=1012) imr_spx,imr_spy
c              write(*,*) '...size pix: ',imr_spx,imr_spy
          endif
          if(string(1:4).eq.'dist') then
              read(2,*,end=1012,err=1012) distance
              read(2,*,end=1001,err=1001) string
              if((string(1:2).eq.'AU').or.
     %            (string(1:2).eq.'au')) then
                  distance = 4.848139d-6 * distance
              elseif((string(1:2).eq.'PC').or.
     %               (string(1:2).eq.'pc').or.
     %               (string(1:6).eq.'parsec')) then
                  distance = 1.d0 * distance
              elseif(string(1:2).eq.'cm') then
                  distance = 3.2407347d-19 * distance
              else
                  strread = .true.
                  goto 1001
              endif              
          endif
          if(string(1:9).eq.'imagesize') then
              read(2,*,end=1012,err=1012) imr_szimx,imr_szimy
              read(2,*,end=1066,err=1012) szunit
              goto 1067
 1066         continue
              write(*,*) 'Sorry, must specify size unit'
              stop 13
 1067         continue
              if((szunit(1:2).eq.'AU').or.
     %            (szunit(1:2).eq.'au')) then
                  scalefact = 1.496d13
              elseif((szunit(1:2).eq.'PC').or.
     %               (szunit(1:2).eq.'pc').or.
     %               (szunit(1:6).eq.'parsec')) then
                  scalefact = 3.08572d18
              elseif(szunit(1:2).eq.'cm') then
                  scalefact = 1.d0
              elseif(szunit(1:6).eq.'arcsec') then
                  iskyangle = 1
                  scalefact = 1.496d13
              else
                  write(*,*) 'Sorry, do not know unit type ',szunit
                  stop 13
              endif
c              write(*,*) '...size image: ',imr_szimx,imr_szimy
          endif
          if(string(1:6).eq.'offset') then
              read(2,*,end=1012,err=1012) imr_xoff,
     %                           imr_yoff,imr_phioff
c              write(*,*) '...offsets: ',imr_xoff,
c     %                           imr_yoff,imr_phioff
          endif
          goto 1001
 1012     continue
          write(*,*) 'WARNING: Wanted to read data from the ',
     %        'command file, but somehow could not do it...'
          write(*,*) 'Using defaults'
 1013     continue
          if((imseq_inu0.lt.1).or.(imseq_inu1.gt.freq_nr)) then
              write(*,*) 'Sorry, but in the command file you'
              write(*,*) '       specified inconsistent '
              write(*,*) '       inu0 or inu1'
              stop 13
          endif
c
c         Scale the image size
c
          if(iskyangle.eq.0) then
              imr_szimx = scalefact * imr_szimx
              imr_szimy = scalefact * imr_szimy
          else
              if(distance.gt.0.d0) then
                  imr_szimx = scalefact * distance * imr_szimx
                  imr_szimy = scalefact * distance * imr_szimy
              else
                  write(*,*) 'Sorry, when you specify ',szunit
                  write(*,*) 'you also must specify the ',
     %                 'distance in parsec...'
                  stop 13
              endif
          endif
c
c         Specify pixel size
c
          if(imr_szimx.gt.0.d0) then
              imr_spx = imr_szimx / (0.5d0*imr_nx)
          endif
          if(imr_szimy.gt.0.d0) then
              imr_spy = imr_szimy / (0.5d0*imr_ny)
          endif
c
      endif
c
c     Done reading the command file
c     
 995  continue
      close(2)
      end



c     --------------------------------------------------------------
c                     READ THE GRID AND SETUP
c     --------------------------------------------------------------
      subroutine read_all()
      implicit none
      doubleprecision dummy,frq,dum
      integer i,ir,it,nr,nt,ix,iy,itmax,k,j,imirt,inu
      integer iformat,isize,ispec,nnf,idum,nf
      logical fex
c
#include "common_grid.h"
#include "common_dust.h"
#include "common_main.h"
c
c     Read the R grid
c     
      open(unit=3,file='radius.inp',status='old')
      read(3,*) irsi_frsizex
      if(irsi_frsizex.gt.FRSIZE_X) then
          write(*,*) 'ERROR: radius.inp has more radial gridpoints'
          write(*,*) '       than FRSIZE_X. '
          write(*,*) '       Recompile with larger FRSIZE_X'
          stop 13
      endif
      do ix=1,irsi_frsizex
          read(3,*) rsi_x_c(ix,1)
      enddo
      close(3)
c     
c     The boundaries
c
      rsi_x_c(0,1)  = rsi_x_c(1,1)**2 / rsi_x_c(2,1)
      rsi_x_c(-1,1) = rsi_x_c(0,1)**2 / rsi_x_c(1,1)
      rsi_x_c(irsi_frsizex+1,1) = rsi_x_c(irsi_frsizex,1)**2 
     %     / rsi_x_c(irsi_frsizex-1,1)
      rsi_x_c(irsi_frsizex+2,1) = rsi_x_c(irsi_frsizex+1,1)**2 
     %     / rsi_x_c(irsi_frsizex,1)
c
c     The cell interfaces
c
      do ix=0,irsi_frsizex+2
          rsi_x_i(ix,1)  = sqrt(rsi_x_c(ix,1)*rsi_x_c(ix-1,1))
      enddo
c
c     Read Theta grid from file, and recompute all other data. 
c     Beware of the theta mirroring option.
c     
      open(unit=3,file='theta.inp',status='old')
      read(3,*) irsi_frsizey,imirt
      itmax = irsi_frsizey
      do iy=1,itmax
          read(3,*) rsi_x_c(iy,2)
          if(rsi_x_c(iy,2).lt.0.d0) then
              write(*,*) 'ERROR: theta.inp has negative values.'
              stop 13                  
          endif
      enddo
      if(imirt.ne.0) then
          irsi_frsizey = 2*irsi_frsizey
      endif
      if(irsi_frsizey.gt.FRSIZE_Y) then
          write(*,*) 'ERROR: theta.inp has more Theta gridpoints'
          write(*,*) '       than FRSIZE_Y. '
          write(*,*) '       Recompile with larger FRSIZE_Y'
          stop 13
      endif
      if(imirt.ne.0) then
          do iy=1,itmax
              rsi_x_c(2*itmax+1-iy,2) = 
     %             3.14159265359d0 - rsi_x_c(iy,2)
          enddo      
      endif
      close(3)
c
c     Do boundaries right
c
      rsi_x_c(0,2)  = - rsi_x_c(1,2)
      rsi_x_c(-1,2) = - rsi_x_c(2,2)
      rsi_x_c(irsi_frsizey+1,2) = 2 * 3.1415926e0 
     %                  - rsi_x_c(irsi_frsizey,2)
      rsi_x_c(irsi_frsizey+2,2) = 2 * 3.1415926e0 
     %                  - rsi_x_c(irsi_frsizey-1,2)
c         
c     Cell interfaces
c         
      do iy=0,irsi_frsizey+2
          rsi_x_i(iy,2)  = 0.5d0 * ( rsi_x_c(iy-1,2) + 
     %         rsi_x_c(iy,2) )
      enddo
c
c     Read the frequency
c     
      open(unit=1,file='frequency.inp',status='old')
      read(1,*) freq_nr
      if(freq_nr.gt.FRSIZE_FREQ) then
          write(*,*) 'ERROR: frequency.inp has more gridpoints'
          write(*,*) '       than FRSIZE_FREQ. '
          write(*,*) '       Recompile with larger FRSIZE_FREQ'
          stop 13
      endif          
      do inu=1,freq_nr
          read(1,*) freq_nu(inu)
      enddo
      close(1)
c
c     Read the dust opacities
c
      call read_dustdata()
c
c     Read the dust density
c
      open(unit=1,file='dustdens.inp',status='old')
      read(1,*) k,nr,nt,j
      if((nr.ne.irsi_frsizex).or.(nt.ne.irsi_frsizey/2)) stop 721
      if(k.ne.dust_nr_species) then
          write(*,*) 'ERROR: In the dustopac.inp file the number'
          write(*,*) '  of dust species is different from that '
          write(*,*) '  specified in dustdens.inp'
          stop 13
      endif
      do ispec=1,dust_nr_species
          do ir=1,nr
              do it=1,nt
                  read(1,*) dum
                  if(dum.gt.1d-90) then
                      dust_rho(ispec,it,ir) = dum
                  else
                      dust_rho(ispec,it,ir) = 1d-90
                  endif
              enddo
          enddo
      enddo
      close(1)
c
c     Read the dust temperature
c
      open(unit=1,file='dusttemp_final.dat',status='old')
      read(1,*) k,nr,nt,j
      if((nr.ne.irsi_frsizex).or.(nt.ne.irsi_frsizey/2)) stop 721
      if(k.ne.dust_nr_species) then
          write(*,*) 'ERROR: In the dustopac.inp file the number'
          write(*,*) '  of dust species is different from that '
          write(*,*) '  specified in dusttemp_final.dat'
          stop 13
      endif
      do ispec=1,dust_nr_species
          read(1,*) k
          if(k.ne.dust_nr_size(ispec)) then
              write(*,*) 'ERROR: In the dustopac.inp file the number'
              write(*,*) '  of dust sizes for this dust species ',
     %                    ispec
              write(*,*) '  is different from that '
              write(*,*) '  specified in dusttemp_final.dat'
              stop 13
          endif
          do isize=1,dust_nr_size(ispec)
              do ir=1,nr
                  do it=1,nt
                      read(1,*) dum
                      if(dum.gt.1d-90) then
                          dust_temp(isize,ispec,it,ir) = dum
                      else
                          dust_temp(isize,ispec,it,ir) = 1d-90
                      endif
                  enddo
              enddo
          enddo
      enddo
      close(1)
c
c     Read the stellar information
c
      open(unit=1,file='starinfo.inp',status='old')
      read(1,*) iformat
      read(1,*) radbnd_rstar
      close(1)
      open(unit=3,file='starspectrum.inp',status='old')
      read(3,*) i
      if(i.ne.freq_nr) then
          write(*,*) 'Sorry, stellar spectrum in starspectrum.inp'
          write(*,*) 'does not have the same number of freqs as ',
     %         'other input files.'
          stop 13
      endif
      if(i.eq.1) then
          read(3,*) frq,dummy
          radbnd_starspec(1) = 3.0308410d36 * 
     %         dummy / radbnd_rstar**2
      else
          do i=1,freq_nr
              read(3,*) frq,dummy 
              if(abs(frq-freq_nu(i))/(frq+freq_nu(i)).gt.1.d-3) then
                  write(*,*) 'PROBLEM: Frequency grid of stellar ',
     %                     'spectrum unequal to frequency.inp'
                  write(*,*) frq,freq_nu(i)
                  stop 13
              endif
c             NOTE: pc^2/pi = 3.0308410d36
              radbnd_starspec(i) = 3.0308410d36 * 
     %             dummy / radbnd_rstar**2
          enddo
          close(3)
      endif
c
c     Compute the total stellar luminosity in case we do the 
c     luminosity conservation test
c
      radbnd_lstar=0.d0
      do inu=1,freq_nr
          radbnd_lstar = radbnd_lstar + 
     %              3.14159265359d0 * dust_freq_wgt(inu) *
     %              radbnd_starspec(inu)
      enddo
      radbnd_lstar = 1.25663706144d1 * 
     %               radbnd_rstar**2 * radbnd_lstar
c
c     Read interstellar radiation field if it is given
c
      inquire(file='interstellfield.inp',exist=fex)
      if(fex) then
          open(unit=1,file='interstellfield.inp',status='old')
          read(1,*) nf
          if(nf.ne.freq_nr) then
              write(*,*) 'ERROR: interstellfield.inp inconsistent nr'
              write(*,*) '    of frequencies compared to frequency.inp'
              stop 13
          endif
          write(*,*) 'Reading interstellarfield.inp...'
          do inu=1,nf
              read(1,*) radbnd_interstellfield(inu)
          enddo
          close(1)
      else
          do inu=1,freq_nr
              radbnd_interstellfield(inu) = 0.d0
          enddo
      endif
c
c     Install the extinction, computed from the dust opacities
c     and the dust densities, including both absorption and 
c     scattering
c
      call compute_alpha()
c
c     Now calculate the source terms from the dust temperature
c
      call src_clear()
      call calc_emis_dust()
c
c     Now add the scattering (in isotropic scattering approximation)
c     to the source terms
c
      call read_scatsource()
c
      end



c     --------------------------------------------------------------
c                      READ THE APERTURE FILE
c
c     This is for the spectrum, if an aperture as a function of 
c     wavelength is requested. This aperture should be given 
c     in arcsecond.
c     --------------------------------------------------------------
      subroutine read_aperture_file()
      implicit none
c
#include "common_grid.h"
#include "common_main.h"
#include "common_telescope.h"
c
      integer nf,inu
      logical fex
c
      inquire(file='aperture.inp',exist=fex)
      if(fex) then
          write(*,*) '***********************'
          write(*,*) 'Reading aperture.inp...'
          write(*,*) '***********************'
          open(unit=1,file='aperture.inp',status='old')
          read(1,*) nf
          if(nf.ne.freq_nr) then
              write(*,*) 'ERROR: Nr of frequency points in aperture.inp'
              write(*,*) '       is inconsistent with rest of the setup'
              stop
          endif
          do inu=1,freq_nr
              read(1,*) spec_aperture(inu)
          enddo
          close(1)
          telesc_aperture = 1
      endif
c
      end


c     ==============================================================
c                          THE PARLIST STUFF
c     ==============================================================

c     --------------------------------------------------------------
c            READ IN THE ENTIRE INPUT FILE INTO A STRING ARRAY
c     --------------------------------------------------------------
      subroutine read_input_file()
      implicit none
c
      common/parses/inpstring
      character*160 inpstring(INPUT_MAX_LEN)
      common/parsei/nrstring,lineok
      integer nrstring,lineok(INPUT_MAX_LEN)
c
      character*160 string
      integer iline,i
c
      do iline=1,INPUT_MAX_LEN
          read(1,'(A158)',end=10) string
          inpstring(iline) = string
          lineok(iline) = 0
      enddo
      write(*,*) 'Input file contains too many lines...'
      stop
 10   continue
      nrstring=iline-1
c      
      end



c     --------------------------------------------------------------
c         BASIC PARSING ROUTINE: SPLIT STRING INTO NAME AND VALUE
c
c     This routine interprets lines such as
c  
c       ; The following variable is useful:
c       var1   = 5.74d0  ; Here some comments
c
c     The first line is simply ignored (though marked as OK) and
c     the second line would be split into strname='var1' and
c     strvalue='5.74d0' with lenname being 4 and lenvalue being 6.
c     
c     --------------------------------------------------------------
      subroutine parse_name_value(string,strname,lenname,
     %                                   strvalue,lenvalue,icomm)
      implicit none
      character*160 string
      character*80 strname,strvalue
      integer lenname,lenvalue
      logical icomm
c
      integer ilen,i,istart,istop
c
      icomm    = .false.
      strname  = ''
      lenname  = 0
      strvalue = ''
      lenvalue = 0
      ilen     = len_trim(string)
      istart   = 1
      do i=1,ilen-1
          if(string(i:i).eq.' ') then
              istart=i+1
          else
              goto 20
          endif
      enddo
      icomm=.true.
      return
 20   continue
      if(string(istart:istart).eq.';') then
          icomm=.true.
          return
      endif
      if(istart.ge.ilen) return
      do i=istart+1,ilen-1
          if(string(i:i).eq.' ') goto 40
          if(string(i:i).eq.'=') goto 40
      enddo
      return
 40   continue
      istop=i-1
      lenname = istop-istart+1
      if(lenname.gt.80) then
          write(*,*) 'Variable name too long'
          stop
      endif
      strname = string(istart:istop)
      do i=istop+1,ilen-1
          if(string(i:i).eq.'=') goto 50
      enddo
      lenname=0
      strname=''
      lenvalue=0
      strvalue='' 
      return
 50   continue
      istart = i+1
      if(istart.gt.ilen) stop
      do i=istart,ilen
          if(string(i:i).eq.' ') then
              istart=i+1
          else
              goto 60
          endif
      enddo
 60   continue
      do i=istart+1,ilen
          if(string(i:i).eq.';') goto 80
          if(string(i:i).eq.' ') goto 80
      enddo
 80   continue
      istop = i-1
      lenvalue = istop-istart+1
      if(lenvalue.gt.80) then
          write(*,*) 'Value length too long'
          stop
      endif
      strvalue = string(istart:istop)
c
c      write(*,*) '##',strname(1:lenname),'##',strvalue(1:lenvalue),'##'
c     
      end


c     --------------------------------------------------------------
c                         STRING COMPARING
c     --------------------------------------------------------------
      function stringcompare(string1,string2,len)
      implicit none
      character*80 string1,string2
      integer len
      integer i
      logical stringcompare
c
      if(len.gt.80) stop
      if(len.le.0) then
          stringcompare=.false.
          return
      endif
      stringcompare=.true.
      do i=1,len
          if(string1(i:i).ne.string2(i:i)) stringcompare=.false.
      enddo
      return
      end


c     --------------------------------------------------------------
c                       PARSING ROUTINE FOR DOUBLE
c     --------------------------------------------------------------
      subroutine parse_input_double(name,value)
      implicit none
      character*80 name
      doubleprecision value
c
      common/parses/inpstring
      character*160 inpstring(INPUT_MAX_LEN)
      common/parsei/nrstring,lineok
      integer nrstring,lineok(INPUT_MAX_LEN)
c
      integer iline,ichar
      character*80 strname,strvalue
      integer lenname,lenvalue,lenn
      logical stringcompare,found,icomm
c
      found=.false.
      do ichar=2,80
          if(name(ichar:ichar).eq.'@') goto 30
      enddo
      write(*,*) 'INTERNAL ERROR: Keywords must end with @!!!'
      stop
 30   continue
      lenn=ichar-1
      do iline=1,nrstring
          call parse_name_value(inpstring(iline),
     %             strname,lenname,strvalue,lenvalue,icomm)
          if(icomm) then
              lineok(iline) = 1
          elseif(stringcompare(strname,name,lenn).and.
     %           (lenn.eq.lenname)) then
              if(found) then
                  write(*,*) 'Found one parameter more than once'
                  write(*,*) 'in the input file: ',strname(1:lenname)
                  write(*,*) '*** ABORTING ***'
                  stop
              endif
              read(strvalue(1:lenvalue),*) value
              found=.true.
              lineok(iline) = 1
          endif
      enddo
c
      end

c     --------------------------------------------------------------
c                       PARSING ROUTINE FOR INTEGER
c     --------------------------------------------------------------
      subroutine parse_input_integer(name,value)
      implicit none
      character*80 name
      integer value
c
      common/parses/inpstring
      character*160 inpstring(INPUT_MAX_LEN)
      common/parsei/nrstring,lineok
      integer nrstring,lineok(INPUT_MAX_LEN)
c
      integer iline,ichar
      character*80 strname,strvalue
      integer lenname,lenvalue,lenn
      logical stringcompare,found,icomm
c
      found=.false.
      do ichar=2,80
          if(name(ichar:ichar).eq.'@') goto 30
      enddo
      write(*,*) 'INTERNAL ERROR: Keywords must end with @!!!'
      stop
 30   continue
      lenn=ichar-1
      do iline=1,nrstring
          call parse_name_value(inpstring(iline),
     %             strname,lenname,strvalue,lenvalue,icomm)
          if(icomm) then
              lineok(iline) = 1
          elseif(stringcompare(strname,name,lenn).and.
     %           (lenn.eq.lenname)) then
              if(found) then
                  write(*,*) 'Found one parameter more than once'
                  write(*,*) 'in the input file: ',strname(1:lenname)
                  write(*,*) '*** ABORTING ***'
                  stop
              endif
              read(strvalue(1:lenvalue),*) value
              found=.true.
              lineok(iline) = 1
          endif
      enddo
c
      end

c     --------------------------------------------------------------
c                   CHECK IF ALL LINES ARE OK
c     --------------------------------------------------------------
      subroutine check_all_lines_ok(ierror)
      implicit none
      logical ierror
c
      common/parses/inpstring
      character*160 inpstring(INPUT_MAX_LEN)
      common/parsei/nrstring,lineok
      integer nrstring,lineok(INPUT_MAX_LEN)
c
      character*160 string
      integer iline,ilen
c
      ierror = .false.
      do iline=1,nrstring
          if(lineok(iline).eq.0) then
              write(*,*) 'ERROR in input file: variable unknown:'
              ilen = len_trim(inpstring(iline))
              string = inpstring(iline)
              write(*,*) string(1:ilen)
              ierror = .true.
          endif
      enddo
c
      end
