c===================================================================
c                        RADICAL 2-D 
c
c                          (C) 1999
c
c               C.P. Dullemond  and  R. Turolla
c
c
c This is the configuration file for RADICAL. This file is made such 
c that by reading carefully through the file from top to bottom, and 
c following the directives, one should be able to configurate the 
c code to the desired form. 
c===================================================================
c
#include "configure.h"
c
c===================================================================
c                        SYSTEM SPECIFIC STUFF
c===================================================================
c
#define OPENACC access
c
c-------------------------------------------------------------------
c Specific to the XL Fortran compiler for AIX
c-------------------------------------------------------------------
c
#ifdef COMPILER_XLAIX
#undef OPENACC
#define OPENACC position
#define sign sgnf
#endif
c
c-------------------------------------------------------------------
c First check if the most important defines are indeed defined
c-------------------------------------------------------------------
c
#ifndef FRSIZE_X
#  error : Need to define FRSIZE_X
#endif
#ifndef FRSIZE_Y
#  error : Need to define FRSIZE_Y
#endif
#ifndef COORD_SPHERICAL
#   ERROR: Must specify COORD_SPHERICAL for now
#endif
c
c-------------------------------------------------------------------
c Check the dimensions 
c-------------------------------------------------------------------
c
#define RADGRID_TWODIM
#if (FRSIZE_Y+2)/2==(FRSIZE_Y+1)/2
#       ERROR:  FRSIZE_Y SHOULD BE EVEN
#endif
#if (FRSIZE_PHI+2)/2==(FRSIZE_PHI+1)/2
#       ERROR: FRSIZE_PHI SHOULD BE EVEN
#else
#if (FRSIZE_PHI/2+2)/2==(FRSIZE_PHI/2+1)/2
#       ERROR: FRSIZE_PHI NOT DIVISIBLE BY 4
#endif
#endif
c
c-------------------------------------------------------------------
c Check interpolation schemes
c-------------------------------------------------------------------
c
#ifdef INTERPOL_ORDER_1
#ifdef INTERPOL_THETA_3
#error : FIRST ORDER, SO NO THETA_3
#endif
#ifdef INTERPOL_PHI_3
#error : FIRST ORDER, SO NO PHI_3
#endif
#endif
c
c===================================================================
c                end of consistency checks
c===================================================================
c
#define NO_SHORT_CHARS
#define NO_INTENSITY_STORAGE
c
c-------------------------------------------------------------------
c Then we must take care of the mirroring and stuff.
c-------------------------------------------------------------------
c
#ifndef MIRROR_THETA
#define FRSIZE_Y_SMALL FRSIZE_Y
#else
#define FRSIZE_Y_SMALL (FRSIZE_Y/2)
#endif
#ifndef MIRROR_PHI
#define FRSIZE_PHI_SMALL FRSIZE_PHI
#else
#define FRSIZE_PHI_SMALL (FRSIZE_PHI/2)
#endif
#define FRSIZE_INF_PHI_SMALL FRSIZE_INF_PHI
c
c-------------------------------------------------------------------
c For small memory: all frequencies use same memory
c-------------------------------------------------------------------
c 
#ifdef SMALL_MEMORY
# define FRSIZE_INF_FREQ 1
#else
# define FRSIZE_INF_FREQ FRSIZE_FREQ
#endif
c
c===================================================================
c               end of memory saving defines
c===================================================================
c
c-------------------------------------------------------------------
c Dust defines
c-------------------------------------------------------------------
c
#define DUST_RADICAL
#ifndef DUST_TRANGE_MAX
#define DUST_TRANGE_MAX 1
#endif
#define DUST_RHOMIN 1d-99
c
c
c-------------------------------------------------------------------
c Define for grid setup stuff
c-------------------------------------------------------------------
c
#define MUTHRES 1.d-6
#define FRSIZE_MU_HALF (FRSIZE_MU/2)
c
c-------------------------------------------------------------------
c Frequency grid define (for dust tridiagonal ALI, future).
c-------------------------------------------------------------------
c
#ifndef DUST_TRIDAG_ALI
#define FREQ_START 1
#else
#define FREQ_START 0
#endif
#ifndef DUST_TEMP_MIN 
#define DUST_TEMP_MIN 0.02d0
#endif
#ifndef DUST_TEMP_MAX 
#define DUST_TEMP_MAX 3000.d0
#endif
#ifndef DUST_TEMP_ACCUR
#define DUST_TEMP_ACCUR 0.01d0
#endif
c
c-------------------------------------------------------------------
c Version number
c-------------------------------------------------------------------
c
#define VERSION "V. 0.3"
c
c-------------------------------------------------------------------
c Find the maximum dimension
c-------------------------------------------------------------------
c
#if FRSIZE_X > FRSIZE_Y           /* Be sure to have the correct one... */
#   define FRSIZE_MAX FRSIZE_X    /* This is the biggest mesh size. */
#else
#   define FRSIZE_MAX FRSIZE_Y    /* This is the biggest mesh size. */
#endif
c
c-------------------------------------------------------------------
c Some constants
c-------------------------------------------------------------------
c
#define LOWER_DOUBLE (1e-33)
#define E_LOW       (1e-12)
#define PICONST 3.1415926535897932385d0
#define TEMPCMB 2.728d0
#define SYNCH_LOWER_T_CUTOFF (5.d8)
c
c-------------------------------------------------------------------
c Stuff for the interpolations
c-------------------------------------------------------------------
c
#ifdef MIRROR_THETA
#define IDX0 0
#define IDX1 1
#define IDX2 2
#define IDX3 3
#else
#define IDX0 0
#define IDX1 0
#define IDX2 0
#define IDX3 0
#endif
c
c-------------------------------------------------------------------
c TELESCOPE/LONGCHAR settings
c-------------------------------------------------------------------
c
#ifndef TELESC_EPS
#define TELESC_EPS 1.d-6
#endif
